@@ -980,7 +980,7 @@ def create_group(resource: Dict):
             # Get the role
             role = role_item.get(PROP_ROLE)
             if role is None:
-                raise Exception("Role must have 'role' property")
+                raise Exception("Role must have 'role' specified")
 
             # Get the ID and name of the role
             id_ = role.get(PROP_ID)
@@ -1035,13 +1035,15 @@ def create_group(resource: Dict):
 
         return role_specifications
 
-    def add_or_update_roles(role_specifications: List[RoleSpecification]):
+    def add_or_update_roles(
+        group_id_: str, role_specifications: List[RoleSpecification]
+    ):
         """
         Helper function to add/update a list of roles.
         """
         for role_spec in role_specifications:
             CLIENT.account_client.add_role_to_group(
-                group_id,
+                group_id_,
                 role_spec.id,
                 RoleScope(role_spec.global_, role_spec.namespaces),
             )
@@ -1054,23 +1056,16 @@ def create_group(resource: Dict):
                     f"namespace(s): {', '.join(ns_list_quoted)}"
                 )
 
-    def remove_roles(role_specifications: List[RoleSpecification]):
+    def remove_roles(group_id_: str, role_specifications: List[RoleSpecification]):
         """
         Helper function to remove a list of roles.
         """
         for role_spec in role_specifications:
             CLIENT.account_client.remove_role_from_group(
-                group_id,
+                group_id_,
                 role_spec.id,
             )
-            if role_spec.global_:
-                print_log(f"Removed role '{role_spec.name}' with global scope")
-            else:
-                ns_list_quoted = [f"'{ns}'" for ns in role_spec.namespaces]
-                print_log(
-                    f"Removed role '{role_spec.name}' scoped to "
-                    f"namespace(s): {', '.join(ns_list_quoted)}"
-                )
+            print_log(f"Removed role '{role_spec.name}'")
 
     def get_roles_to_remove(
         existing_roles: List[GroupRole], new_roles: List[RoleSpecification]
@@ -1098,40 +1093,42 @@ def create_group(resource: Dict):
             if role_spec.name not in [role_spec.name for role_spec in new_roles]
         ]
 
-    def add_group() -> str:
+    def add_group() -> Group:
         """
         Helper function to add a new group.
         Return the ID of the newly created group.
         """
-        group: Group = CLIENT.account_client.add_group(
+        group_: Group = CLIENT.account_client.add_group(
             AddGroupRequest(name=name, description=description)
         )
-        print_log(f"Created Group '{group.name}' ({group.id})")
+        print_log(f"Created Group '{group_.name}' ({group_.id})")
         clear_group_caches()
-        return group.id
+        return group_
 
-    def update_group(group_id: str):
+    def update_group(group_id_: str) -> Optional[Group]:
         """
         Helper function to update an existing group, including updating
         its roles.
         """
-        if not confirmed(f"Update Group '{name}' ({group_id})?"):
-            return
-        group: Group = CLIENT.account_client.update_group(
-            group_id, UpdateGroupRequest(name=name, description=description)
+        if not confirmed(f"Update Group '{name}' ({group_id_})?"):
+            return None
+        group_: Group = CLIENT.account_client.update_group(
+            group_id_, UpdateGroupRequest(name=name, description=description)
         )
-        print_log(f"Updated Group '{group.name}' ({group.id})")
-        updated_role_specs = get_updated_role_specifications()
-        remove_roles(get_roles_to_remove(group.roles, updated_role_specs))
-        add_or_update_roles(updated_role_specs)
+        print_log(f"Updated Group '{group_.name}' ({group_.id})")
+        return group_
 
     # Main logic
     group_id = get_group_id_by_name(CLIENT, name)
     if group_id is None:  # New group
-        group_id = add_group()
-        add_or_update_roles(get_updated_role_specifications())
+        group = add_group()
+        add_or_update_roles(group.id, get_updated_role_specifications())
     else:  # Existing group
-        update_group(group_id)
+        group = update_group(group_id)
+        if group is not None:
+            updated_role_specs = get_updated_role_specifications()
+            add_or_update_roles(group_id, updated_role_specs)
+            remove_roles(group_id, get_roles_to_remove(group.roles, updated_role_specs))
 
 
 def create_application(resource: Dict):
@@ -1144,25 +1141,20 @@ def create_application(resource: Dict):
     except KeyError as e:
         raise Exception(f"Expected property to be defined ({e})")
 
-    groups: Optional[List[str]] = resource.pop(PROP_GROUPS, None)
-
-    if groups is not None:
-        # Convert group names to IDs
-        new_group_ids = set()
-        for group_name in groups:
-            app_id = get_group_id_by_name(CLIENT, group_name)
-            if app_id is None:
-                print_warning(f"Group '{group_name}' not found ... ignoring")
-            else:
-                new_group_ids.add(app_id)
+    groups: List[str] = resource.pop(PROP_GROUPS, [])
+    # Convert group names to IDs
+    new_group_ids = set()
+    for group_name in groups:
+        app_id = get_group_id_by_name(CLIENT, group_name)
+        if app_id is None:
+            print_warning(f"Group '{group_name}' not found ... ignoring")
+        else:
+            new_group_ids.add(app_id)
 
     def update_groups(app: Application):
         """
         Helper function to add/remove groups from an application.
         """
-        if groups is None:
-            return
-
         current_group_ids = {
             group.id for group in get_application_groups(CLIENT, app.id)
         }
@@ -1259,34 +1251,29 @@ def update_user(resource: Dict, internal_user: bool):
             f"resource '{RN_EXTERNAL_USER}' ({resource})"
         )
 
-    groups: Optional[List[str]] = resource.pop(PROP_GROUPS, None)
+    groups: List[str] = resource.pop(PROP_GROUPS, [])
+    new_group_ids = set()
+    # Convert group names to IDs
+    for group_name in groups:
+        group_id = get_group_id_by_name(CLIENT, group_name)
+        if group_id is None:
+            print_warning(f"Group '{group_name}' not found ... ignoring")
+        else:
+            new_group_ids.add(group_id)
 
-    if groups is not None:
-        # Convert group names to IDs
-        new_group_ids = set()
-        for group_name in groups:
-            group_id = get_group_id_by_name(CLIENT, group_name)
-            if group_id is None:
-                print_warning(f"Group '{group_name}' not found ... ignoring")
-            else:
-                new_group_ids.add(group_id)
-
-    def update_groups(user: User):
+    def update_groups():
         """
         Helper function to add/remove groups from a user.
         """
-        if groups is None:
-            return
-
-        if not confirmed(f"Update Groups for User '{username}' ({user.id})?"):
-            return
-
         current_group_ids = {group.id for group in get_user_groups(CLIENT, user.id)}
 
         if current_group_ids == new_group_ids:
             print_log("No Group additions or deletions required")
             return
 
+        if not confirmed(f"Update Groups for User '{username}' ({user.id})?"):
+            return
+
         group_ids_to_remove = current_group_ids - new_group_ids
         for group_id in group_ids_to_remove:
             CLIENT.account_client.remove_user_from_group(group_id, user.id)
@@ -1323,11 +1310,8 @@ def update_user(resource: Dict, internal_user: bool):
         return
 
     username = user.username if isinstance(user, InternalUser) else user.name
-
-    if groups is not None:
-        update_groups(user)
-    else:
-        print_log(f"Nothing to do for User '{username}' ({user.id})")
+    update_groups()
+    print_log(f"Actions complete for User '{username}' ({user.id})")
 
 
 def create_namespace(resource: Dict):
